# -*- encoding: utf-8 -*-

#MMC_LOCK_TYPE = 'RedisLock'

from settings import INSTALLED_APPS

'''
INSTALLED_APPS += ('raven.contrib.django.raven_compat',)
#

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'root': {
        'level': 'WARNING',
        'handlers': ['sentry'],
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
    },
    'handlers': {
        'sentry': {
            'level': 'ERROR',
            'class': 'raven.contrib.django.raven_compat.handlers.SentryHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
    },
}

RAVEN_CONFIG = {
    'dsn': 'https://1c66b94401184cba8c2c3370b5cc6971:11a19e5e6f1c417a928952eee4434fa9@app.getsentry.com/26104',
}
'''
import sys
import django

#print sys.version
#print '>>', django.get_version()

if 'test' not in sys.argv:
    MMC_SUBJECT = '[MMC] Error: %(host)s: %(script)s'
    if django.VERSION[:2] < (1, 7):
        INSTALLED_APPS += ('south',)



from django.contrib import admin
try:
    from django.conf.urls import patterns, include, url
except:
    from django.conf.urls.defaults import patterns, include, url

admin.autodiscover()

urlpatterns = patterns(
    '',
    url(r'^admin/', include(admin.site.urls)),
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'MMCHost'
        db.create_table('mmc_mmchost', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateField')(auto_now=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('mmc', ['MMCHost'])

        # Adding model 'MMCScript'
        db.create_table('mmc_mmcscript', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateField')(auto_now=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal('mmc', ['MMCScript'])

        # Adding model 'MMCLog'
        db.create_table('mmc_mmclog', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('created', self.gf('django.db.models.fields.DateField')(auto_now=True, blank=True)),
            ('start', self.gf('django.db.models.fields.DateTimeField')()),
            ('end', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('elapsed', self.gf('django.db.models.fields.FloatField')()),
            ('hostname', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['mmc.MMCHost'])),
            ('script', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['mmc.MMCScript'])),
            ('success', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('error_message', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('traceback', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('sys_argv', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
        ))
        db.send_create_signal('mmc', ['MMCLog'])


    def backwards(self, orm):
        
        # Deleting model 'MMCHost'
        db.delete_table('mmc_mmchost')

        # Deleting model 'MMCScript'
        db.delete_table('mmc_mmcscript')

        # Deleting model 'MMCLog'
        db.delete_table('mmc_mmclog')


    models = {
        'mmc.mmchost': {
            'Meta': {'object_name': 'MMCHost'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'mmc.mmclog': {
            'Meta': {'object_name': 'MMCLog'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'elapsed': ('django.db.models.fields.FloatField', [], {}),
            'end': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'error_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'hostname': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['mmc.MMCHost']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'script': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['mmc.MMCScript']"}),
            'start': ('django.db.models.fields.DateTimeField', [], {}),
            'success': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'sys_argv': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'traceback': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'})
        },
        'mmc.mmcscript': {
            'Meta': {'object_name': 'MMCScript'},
            'created': ('django.db.models.fields.DateField', [], {'auto_now': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        }
    }

    complete_apps = ['mmc']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mmc', '0002_mmclog_was_notified'),
    ]

    operations = [
        migrations.AddField(
            model_name='mmclog',
            name='stdout_messages',
            field=models.TextField(null=True, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-

from django.core.management import call_command
from django.test import TestCase
from django.core import mail

from mmc.models import MMCScript, MMCLog, MMCHost, MMCEmail
from mmc.defaults import SUBJECT


class MMCTestCase(TestCase):
    def setUp(self):
        MMCEmail.objects.create(email='root@local.host')

    def __test_command(self, command, success):
        call_command(command)

        script = MMCScript.objects.filter(name=command)
        host = MMCHost.objects.all()[0]
        log = MMCLog.objects.filter(script=script, hostname=host)

        self.assertTrue(script.exists())
        self.assertTrue(log.exists())
        self.assertEquals(log.count(), 1)
        self.assertEquals(log[0].success, success)

    def __test_command_ignore(self, command):
        script = MMCScript.objects.create(
            name=command, ignore=True, save_on_error=False)

        call_command(command)

        log = MMCLog.objects.filter(script=script)
        self.assertFalse(log.exists())

    def __check_email(self):
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(mail.outbox[0].subject, SUBJECT)

    # store log
    def test_a_test_command(self):
        self.__test_command('test_command', True)

    def test_b_test_command_noargs(self):
        self.__test_command('test_command_noargs', True)

    def test_c_test_command_error(self):
        self.__test_command('test_command_error', False)
        self.__check_email()

    # ignore log
    def test_a_test_command_ignore(self):
        self.__test_command_ignore('test_command')

    def test_b_test_command_noargs_ignore(self):
        self.__test_command_ignore('test_command_noargs')

    def test_c_test_command_error_ignore(self):
        self.__test_command_ignore('test_command_error')

    # store into log for ignored commands
    def test_d_test_command_error_ignore_save(self):
        command = 'test_command_error'

        MMCScript.objects.create(name=command, ignore=True, save_on_error=True)
        self.__test_command(command, False)
        self.__check_email()



__author__ = 'gotlium'

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def raise_exception(self):
        raise Exception('Hello, World!')

    def handle(self, *args, **options):
        self.raise_exception()



__author__ = 'gotlium'

from django.contrib import admin
from models import MMCLog, MMCScript, MMCHost, MMCEmail


class MMCLogAdmin(admin.ModelAdmin):
    list_display = (
        'script', 'hostname', 'success', 'elapsed',
        'memory', 'cpu_time', 'start', 'end')
    list_filter = ('success', 'hostname', 'script', 'created',)
    list_display_links = ('script',)

    readonly_fields = (
        'start', 'end', 'elapsed', 'hostname', 'script',
        'sys_argv', 'success', 'error_message', 'memory', 'cpu_time')
    fields = (
        'start', 'end', 'elapsed', 'hostname', 'script', 'memory', 'cpu_time',
        'sys_argv', 'success', 'error_message', 'traceback', 'stdout_messages')

    date_hierarchy = 'created'
    ordering = ('-id',)

    search_fields = (
        'error_message',
        'traceback'
        'hostname',
        'script',
    )

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return request.method != 'POST'

    def has_delete_permission(self, request, obj=None):
        return False


class MMCHostAdmin(admin.ModelAdmin):
    list_display = ('name', 'ignore', 'created', 'id',)
    list_filter = ('created', 'ignore',)
    list_display_links = ('name',)

    readonly_fields = ('name',)

    date_hierarchy = 'created'
    ordering = ('-id',)
    search_fields = ('name',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


class MMCScriptAdmin(MMCHostAdmin):
    list_display = (
        'name', 'ignore', 'one_copy', 'save_on_error', 'created', 'id',)
    list_filter = ('ignore', 'one_copy', 'save_on_error')
    fieldsets = [
        ('Basic', {'fields': [
            'ignore', 'one_copy', 'save_on_error'
        ]}),
        ('Triggers', {'fields': [
            'enable_triggers', 'trigger_time', 'trigger_memory', 'trigger_cpu'
        ]}),
    ]


class MMCEmailAdmin(admin.ModelAdmin):
    list_display = ('email', 'is_active', 'created', 'id',)
    list_filter = ('created', 'is_active',)
    list_display_links = ('email',)
    filter_horizontal = ('ignore',)

    date_hierarchy = 'created'
    ordering = ('-id',)
    search_fields = ('email',)


admin.site.register(MMCLog, MMCLogAdmin)
admin.site.register(MMCHost, MMCHostAdmin)
admin.site.register(MMCScript, MMCScriptAdmin)
admin.site.register(MMCEmail, MMCEmailAdmin)



__author__ = 'gotlium'

from time import sleep

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    def handle(self, *args, **options):
        sleep(300)
        print "OK"



__author__ = 'gotlium'

__all__ = ['BaseCommand', 'NoArgsCommand', 'inject_management']

import traceback
import resource
import socket
import atexit
import time
import sys
import os

from django.core.management.base import NoArgsCommand as NoArgsCommandOrigin
from django.core.management.base import BaseCommand as BaseCommandOrigin
from django.utils.encoding import force_unicode
from django.utils.encoding import smart_str
from django.db.utils import DatabaseError

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime

    now = datetime.now

from .defaults import (
    SENTRY_NOTIFICATION, EMAIL_NOTIFICATION, READ_STDOUT, SUBJECT_LIMIT
)
from .lock import get_lock_instance
from .utils import monkey_mix


def mmc_is_test():
    return sys.argv[1:3] == ['test', 'mmc']


class StdOut(object):
    def __init__(self):
        self.data = []

    def write(self, message):
        self.data.append(message)
        try:
            sys.__stdout__.write(message)
        except Exception, msg:
            sys.stderr.write(msg.__str__())

    def __getattr__(self, method):
        def call_method(*args, **kwargs):
            return getattr(sys.__stdout__, method)(*args, **kwargs)

        return call_method

    def get_stdout(self):
        return self.data


class BaseCommandMixin(object):
    def __init__(self):
        if hasattr(self, '_no_monkey'):
            self._no_monkey.__init__(self)
        else:
            super(BaseCommandMixin, self).__init__()

        if READ_STDOUT is True:
            try:
                __IPYTHON__
            except NameError:
                sys.stdout = StdOut()

        self._mmc_start_date = now()
        self._mmc_start_time = time.time()
        self._mmc_elapsed = None
        self._mmc_success = True
        self._mmc_error_message = None
        self._mmc_traceback = None
        self._mmc_show_traceback = False
        self._mmc_script = self.__module__.split('.')[-1]
        self._mmc_lock = get_lock_instance(self._mmc_script)
        self._mmc_exc_info = None
        self._mmc_hostname = socket.gethostname()
        self._mmc_log_instance = None
        self._mmc_resources = resource.getrusage(resource.RUSAGE_SELF)

    def __mmc_one_copy(self):
        try:
            from models import MMCScript

            try:
                if MMCScript.get_one_copy(self._mmc_script):
                    self._mmc_lock.check_is_running()
                    self._mmc_lock.lock()
            except MMCScript.DoesNotExist:
                pass
        except Exception, msg:
            print '[MMC]', msg.__unicode__()

    def __mmc_init(self, *args, **options):
        if not mmc_is_test():
            self.__mmc_one_copy()
            atexit.register(self._mmc_at_exit_callback)
            self._mmc_show_traceback = options.get('traceback', False)

    def __mmc_run(self, *args, **options):
        if hasattr(self, '_no_monkey'):
            self._no_monkey.execute(self, *args, **options)
        else:
            super(BaseCommandMixin, self).execute(*args, **options)

    def __mmc_execute(self, *args, **options):
        try:
            self.__mmc_run(*args, **options)
        except Exception as ex:
            self._mmc_success = False
            self._mmc_error_message = ex.__unicode__()
            self._mmc_traceback = traceback.format_exc()
            self._mmc_exc_info = sys.exc_info()
            if not mmc_is_test():
                raise

    def __mmc_done(self):
        if mmc_is_test():
            self._mmc_at_exit_callback()

    def execute(self, *args, **options):
        self.__mmc_init(*args, **options)
        self.__mmc_log_start()
        self.__mmc_execute(*args, **options)
        self.__mmc_done()

    def __mmc_log_start(self):
        try:
            from mmc.models import MMCLog

            self._mmc_log_instance = MMCLog.logging(
                start=self._mmc_start_date,
                hostname=self._mmc_hostname,
                script=self._mmc_script,
                elapsed=0.00,
                success=None,
                error_message="",
                traceback="",
                sys_argv=' '.join(map(unicode, sys.argv)),
                memory=0.00,
                cpu_time=0.00
            )
        except DatabaseError:
            pass
        except Exception, msg:
            print '[MMC] Logging broken with message:', msg.__unicode__()

    def __mmc_store_log(self):
        try:
            from mmc.models import MMCLog

            memory = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
            resources = resource.getrusage(resource.RUSAGE_SELF)

            utime = resources.ru_utime - self._mmc_resources.ru_utime
            stime = resources.ru_stime - self._mmc_resources.ru_stime
            div = 1024.0 if 'linux' in sys.platform else 1048576.0

            MMCLog.logging(
                instance=self._mmc_log_instance,
                start=self._mmc_start_date,
                hostname=self._mmc_hostname,
                script=self._mmc_script,
                elapsed="%0.2f" % (time.time() - self._mmc_start_time),
                success=self._mmc_success,
                error_message=self._mmc_error_message,
                traceback=self._mmc_traceback,
                sys_argv=' '.join(map(unicode, sys.argv)),
                memory="%0.2f" % (memory / div),
                cpu_time="%0.2f" % (utime + stime),
                stdout_messages=self.__mmc_get_stdout()
            )
        except Exception, msg:
            print '[MMC] Logging broken with message:', msg.__unicode__()

    def __mmc_get_stdout(self):
        if hasattr(sys.stdout, 'get_stdout'):
            try:
                return u''.join(map(force_unicode, sys.stdout.get_stdout()))
            except:
                return repr(sys.stdout.get_stdout())
        return ''

    def __mmc_get_msg_trace(self):
        traceback_msg = ''
        if hasattr(self._mmc_log_instance, 'pk'):
            traceback_msg += '#%d\n\n' % self._mmc_log_instance.pk
        traceback_msg += self._mmc_traceback
        return traceback_msg

    def __mmc_send_mail(self):
        if not self._mmc_success and EMAIL_NOTIFICATION:
            from mmc.models import MMCEmail

            MMCEmail.send(
                self._mmc_hostname, self._mmc_script,
                self.__mmc_get_msg_trace()
            )

    def __mmc_send2sentry(self):
        if not self._mmc_success and SENTRY_NOTIFICATION:
            try:
                from raven.contrib.django.raven_compat.models import client

                client.captureException(
                    exc_info=self._mmc_exc_info, extra=dict(os.environ))
            except:
                pass

    def __mmc_print_log(self):
        if not self._mmc_success and not mmc_is_test():
            if self._mmc_show_traceback:
                print self._mmc_traceback
            else:
                sys.stderr.write(smart_str(
                    'Error: %s\n' % self.style.ERROR(self._mmc_error_message)
                ))
            sys.exit(1)

    def __mmc_notification(self):
        from mmc.models import MMCEmail, MMCLog

        if not self._mmc_log_instance or mmc_is_test():
            return

        cls = MMCLog.objects.get(pk=self._mmc_log_instance.pk)
        if EMAIL_NOTIFICATION and cls.script.enable_triggers is True:
            script = cls.script
            reason = []
            text = ''

            if script.trigger_cpu and cls.cpu_time > script.trigger_cpu:
                reason.append(('cpu', cls.cpu_time, script.trigger_cpu))
            if script.trigger_memory and cls.memory > script.trigger_memory:
                reason.append(('memory', cls.memory, script.trigger_memory))
            if script.trigger_time and cls.elapsed > script.trigger_time:
                reason.append(('time', cls.elapsed, script.trigger_time))

            if reason:
                for data in reason:
                    text += '%s: %f > %f\n' % data

                MMCEmail.send(
                    self._mmc_hostname, self._mmc_script, text, SUBJECT_LIMIT)

    def _mmc_at_exit_callback(self, *args, **kwargs):
        self.__mmc_store_log()
        self._mmc_lock.unlock()
        self.__mmc_notification()
        self.__mmc_send_mail()
        self.__mmc_send2sentry()
        self.__mmc_print_log()


class BaseCommand(BaseCommandMixin, BaseCommandOrigin):
    pass


class NoArgsCommand(BaseCommandMixin, NoArgsCommandOrigin):
    pass


def inject_management():
    monkey_mix(BaseCommandOrigin, BaseCommandMixin)



# -*- encoding: utf-8 -*-

__author__ = 'gotlium'

import datetime

from django.core.management.base import BaseCommand
from django.utils.timezone import now

from mmc.models import MMCLog, MMCEmail


class Command(BaseCommand):
    def handle(self, **options):
        day_ago = (now() - datetime.timedelta(days=1))
        two_day_ago = (day_ago - datetime.timedelta(days=1))
        log_list = MMCLog.objects.filter(
            success__isnull=True, was_notified=False,
            created__range=[two_day_ago, day_ago])

        for log in log_list:
            MMCEmail.send(
                log.hostname.name, log.script.name,
                'Maybe script "%s" was killed by OS kernel.' % log.script.name)
            log.was_notified = True
            log.save()



from setuptools import setup, find_packages
from mmc import get_version


setup(
    name='django-mmc',
    version=get_version(),
    description='App for monitoring management commands on Django.',
    keywords="django management commands",
    long_description=open('README.rst').read(),
    author="GoTLiuM InSPiRiT",
    author_email='gotlium@gmail.com',
    url='https://github.com/LPgenerator/django-mmc',
    packages=find_packages(exclude=['demo']),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)
